from .answer_rephrasing import ANSWER_REPHRASING_PROMPT
from .community import COT_GENERATION_PROMPT, COT_TEMPLATE_DESIGN_PROMPT
from .coreference_resolution import COREFERENCE_RESOLUTION_PROMPT
from .description_rephrasing import DESCRIPTION_REPHRASING_PROMPT
from .kg_extraction import KG_EXTRACTION_PROMPT
from .kg_summarization import KG_SUMMARIZATION_PROMPT
from .multi_hop_generation import MULTI_HOP_GENERATION_PROMPT
from .question_generation import QUESTION_GENERATION_PROMPT
from .search_judgement import SEARCH_JUDGEMENT_PROMPT
from .statement_judgement import STATEMENT_JUDGEMENT_PROMPT
